<?php
/**
 * 2007-2022 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2007-2022 PrestaShop SA
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

if (!defined('_PS_VERSION_')) {
    exit;
}
require_once(_PS_MODULE_DIR_ . '/datapocket/classes/PSWebServiceLibrary.php');

class Datapocket extends Module
{
    protected $config_form = false;

    public $getDatapocketToken;

    public $shopUrl;

    private $webService;

    private $authKey;

    /**
     * @throws PrestaShopWebserviceException
     */
    public function __construct()
    {
        $this->name = 'datapocket';
        $this->tab = 'administration';
        $this->version = '1.0.0';
        $this->author = 'OVIXIA Consultores';
        $this->need_instance = 0;

        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('Datapocket');
        $this->description = $this->l('Having all your assets in one place is critical for teams to increase productivity.
                                     With DataPocket, you can easily locate your entire product catalogue in one place 
                                     and start creating social media posts, posters, logos, presentations and other visual content in a blink.
                                        Design anything and publish anywhere.');

        $this->getDatapocketToken = $this->getDatapocketToken();
        $this->shopUrl = Tools::getHttpHost(true) . __PS_BASE_URI__;
        $this->authKey = $this->getAuthKey();
        $this->webService = new PrestaShopWebservice($this->shopUrl, $this->authKey, false);

        $this->ps_versions_compliancy = array('min' => '1.6', 'max' => _PS_VERSION_);
    }

    public function install()
    {
        Configuration::updateValue('DATAPOCKET_LIVE_MODE', true);

        $this->installDatapocketToken();
        $this->installDB();
        $this->updateDiscounts();

        return parent::install() &&
            $this->registerHook('moduleRoutes') &&
            $this->registerHook('actionProductSave') &&
            $this->registerHook('actionProductUpdate') &&
            $this->registerHook('actionProductDelete') &&
            $this->registerHook('actionCategoryAdd') &&
            $this->registerHook('actionCategoryUpdate') &&
            $this->registerHook('actionCategoryDelete') &&
            $this->registerHook('actionObjectSpecificPriceAddAfter') &&
            $this->registerHook('actionObjectSpecificPriceDeleteAfter') &&
            $this->registerHook('actionObjectSpecificPriceUpdateAfter') &&
            $this->registerHook('actionAdminControllerSetMedia');
    }

    public function updateDiscounts()
    {
        $sql = 'SELECT * FROM `' . _DB_PREFIX_ . 'specific_price`';

        $specificPrices = Db::getInstance()->executeS($sql);

        foreach ($specificPrices as $specificPrice) {
            $sql = 'SELECT `id_datapocket_objects`
                FROM `' . _DB_PREFIX_ . 'datapocket_objects`
                WHERE `id` = ' . $specificPrice['id_specific_price'] . ' AND `object` = "discount"';

            $id_datapocket_objects = Db::getInstance()->getValue($sql);

            if (!$id_datapocket_objects) {
                Db::getInstance()->insert(
                    'datapocket_objects',
                    [
                        'id' => $specificPrice['id_specific_price'],
                        'object' => 'discount',
                        'status' => 'created',
                        'sent' => 0,
                    ]
                );
            }
        }
    }

    public function installDatapocketToken()
    {
        $checkWebservice = Configuration::get('PS_WEBSERVICE');
        if (!$checkWebservice) {
            Configuration::updateValue('PS_WEBSERVICE', true);
        }

        $datapocketToken = Configuration::get('PS_DATAPOCKET_TOKEN');
        if (!$datapocketToken) {
            Configuration::updateValue('PS_DATAPOCKET_TOKEN', Tools::passwdGen(32));
            $datapocketToken = Configuration::get('PS_DATAPOCKET_TOKEN');
        }

        return $datapocketToken;
    }

    public function getAuthKey()
    {
        $sql = 'SELECT `key`
                FROM `' . _DB_PREFIX_ . 'webservice_account`
                WHERE `active` = 1 
                ORDER BY `id_webservice_account`
                DESC';

        return Db::getInstance()->getValue($sql);
    }

    public function uninstall()
    {
        return parent::uninstall();
    }

    public function installDB()
    {
        $sql = [];

        $sql[] = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'datapocket_routes` (
                        `id_datapocket_routes` int(11) NOT NULL AUTO_INCREMENT,
                        `url` varchar(200) NOT NULL,
                        PRIMARY KEY  (`id_datapocket_routes`)
                    ) ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=utf8;';

        $sql[] = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'datapocket_objects` (
                        `id_datapocket_objects` int(11) NOT NULL AUTO_INCREMENT,
                        `id` int(11) NOT NULL,
                        `object` varchar(20) NOT NULL,
                        `status` varchar(20) NOT NULL,
                        `sent` int(0) NOT NULL,
                        PRIMARY KEY  (`id_datapocket_objects`)
                    ) ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=utf8;';

        foreach ($sql as $query) {
            if (!Db::getInstance()->execute($query)) {
                return false;
            }
        }

        return true;
    }

    public function getContent()
    {
        if (((bool)Tools::isSubmit('submitDatapocketModule'))) {
            $this->postProcess();
        }
        $additionalInfo = $this->context->smarty->fetch(
            $this->local_path . 'views/templates/admin/backoffice.tpl'
        );

        return $additionalInfo . $this->renderForm();
    }

    protected function renderForm()
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitDatapocketModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFormValues(), /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );

        return $helper->generateForm(array($this->getConfigForm()));
    }

    protected function getConfigForm()
    {
        return array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('DataPocket settings'),
                    'icon' => 'icon-cogs',
                ),
                'input' => array(
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Live mode'),
                        'name' => 'DATAPOCKET_LIVE_MODE',
                        'is_bool' => true,
                        'desc' => $this->l('Enable/disable DataPocket module'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    array(
                        'type' => 'html',
                        'name' => 'DATAPOCKET_SHOW_TOKEN',
                        'label' => $this->l('DataPocket token'),
                        'html_content' => $this->getDatapocketToken(),
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                ),
            ),
        );
    }

    protected function getConfigFormValues()
    {
        return array(
            'DATAPOCKET_LIVE_MODE' => Configuration::get('DATAPOCKET_LIVE_MODE'),
            'DATAPOCKET_ACCOUNT_PASSWORD' => Configuration::get('DATAPOCKET_ACCOUNT_PASSWORD'),
        );
    }

    protected function getDatapocketToken()
    {
        $this->context->smarty->assign(
            'datapocketToken', Configuration::get('PS_DATAPOCKET_TOKEN')
        );

        return $this->context->smarty->fetch(
            $this->local_path . 'views/templates/admin/getDatapocketToken.tpl'
        );
    }

    public function hookActionAdminControllerSetMedia()
    {
        $this->context->controller->addJs($this->_path.'views/js/datapocket_bo.js');
        $this->context->controller->addCSS($this->_path.'views/js/datapocket_bo.css');
    }

    protected function postProcess()
    {
        $form_values = $this->getConfigFormValues();

        foreach (array_keys($form_values) as $key) {
            Configuration::updateValue($key, Tools::getValue($key));
        }
    }

    public function hookModuleRoutes()
    {
        $slug = 'datapocket';

        return array(
            'module-datapocket-conncheck' => array(
                'controller' => 'conncheck',
                'rule' => $slug . '/conncheck',
                'keywords' => array(),
                'params' => array(
                    'fc' => 'module',
                    'module' => 'datapocket',
                )
            ),
            'module-datapocket-createkey' => array(
                'controller' => 'createkey',
                'rule' => $slug . '/createkey',
                'keywords' => array(),
                'params' => array(
                    'fc' => 'module',
                    'module' => 'datapocket',
                )
            ),
            'module-datapocket-configkey' => array(
                'controller' => 'configkey',
                'rule' => $slug . '/configkey',
                'keywords' => array(),
                'params' => array(
                    'fc' => 'module',
                    'module' => 'datapocket',
                )
            ),
            'module-datapocket-checkkey' => array(
                'controller' => 'checkkey',
                'rule' => $slug . '/checkkey',
                'keywords' => array(),
                'params' => array(
                    'fc' => 'module',
                    'module' => 'datapocket',
                )
            ),
            'module-datapocket-deletekey' => array(
                'controller' => 'deletekey',
                'rule' => $slug . '/deletekey',
                'keywords' => array(),
                'params' => array(
                    'fc' => 'module',
                    'module' => 'datapocket',
                )
            ),
            'module-datapocket-registerwebhook' => array(
                'controller' => 'registerwebhook',
                'rule' => $slug . '/registerwebhook',
                'keywords' => array(),
                'params' => array(
                    'fc' => 'module',
                    'module' => 'datapocket',
                )
            ),
            'module-datapocket-updatewebhook' => array(
                'controller' => 'updatewebhook',
                'rule' => $slug . '/updatewebhook',
                'keywords' => array(),
                'params' => array(
                    'fc' => 'module',
                    'module' => 'datapocket',
                )
            ),
            'module-datapocket-checkwebhook' => array(
                'controller' => 'checkwebhook',
                'rule' => $slug . '/checkwebhook',
                'keywords' => array(),
                'params' => array(
                    'fc' => 'module',
                    'module' => 'datapocket',
                )
            ),
            'module-datapocket-deletewebhook' => array(
                'controller' => 'deletewebhook',
                'rule' => $slug . '/deletewebhook',
                'keywords' => array(),
                'params' => array(
                    'fc' => 'module',
                    'module' => 'datapocket',
                )
            ),
            'module-datapocket-discountdetection' => array(
                'controller' => 'discountdetection',
                'rule' => $slug . '/discountdetection',
                'keywords' => array(),
                'params' => array(
                    'fc' => 'module',
                    'module' => 'datapocket',
                )
            ),
        );
    }

    public function validateToken($token)
    {
        $datapocketToken = Configuration::get('PS_DATAPOCKET_TOKEN');
        if ($datapocketToken == $token) {
            return true;
        }

        return false;
    }

    public function checkEnabledModule()
    {
        $live = Configuration::get('DATAPOCKET_LIVE_MODE');

        if (!$live) {
            die(json_encode([
                'status' => 'error',
                'message' => $this->l('module is disabled')
            ]));
        }
    }

    public function messageNoData()
    {
        http_response_code(401);
        die(json_encode([
            'status' => 'error',
            'message' => $this->l('no data')
        ]));
    }

    public function messageErrorToken()
    {
        http_response_code(401);
        die(json_encode([
            'status' => 'error',
            'message' => $this->l('The provided security token is not valid')
        ]));
    }

    public function frontControllerHeader($get = false)
    {
        ob_clean();
        header_remove();

        header('Content-Type: application/json; charset=utf-8');

        $this->checkEnabledModule();
        if ($get) {
            $data = Tools::getAllValues();
        } else {
            $data = json_decode(file_get_contents('php://input'), true);
        }

        if (!is_array($data)) {
            $this->messageNoData();
        }

        $token = (isset($data['token'])) ? $data['token'] : false;

        if ($token && !$this->validateToken($token)) {
            $this->messageErrorToken();
        }

        return $data;
    }

    public function getWsKeyId($key)
    {
        $sql = 'SELECT `id_webservice_account`
                FROM `' . _DB_PREFIX_ . 'webservice_account`
                WHERE `key` = "' . pSQL($key) . '"';
        $id_webservice_account = Db::getInstance()->getValue($sql);

        if (!$id_webservice_account) {
            http_response_code(404);
            die(json_encode([
                'status' => 'error',
                'message' => $this->l('The Webservice Key does not exist')
            ]));
        }

        return $id_webservice_account;
    }

    public function getWebserviceKeyInfo($wsKey)
    {
        $permissions = WebserviceKey::getPermissionForAccount($wsKey);
        $perms = '';
        if (count($permissions['categories']) > 2) {
            $perms = 'read_write';
        } else {
            $perms = 'read';
        }

        return json_encode([
            'status' => 'success',
            'key' => $wsKey,
            'permissions' => $perms
        ]);
    }

    public function deleteWebserviceKey($id_webservice_account)
    {
        $WebserviceKey = new WebserviceKey($id_webservice_account);

        try {
            if ($WebserviceKey->delete()) {
                return json_encode([
                    'status' => 'success',
                    'message' => $this->l('The key has been removed'),
                    'key' => $WebserviceKey->key,
                ]);
            };
        } catch (Exception $e) {
            return json_encode([
                'status' => 'success',
                'message' => $this->l('The key has been removed'),
            ]);
        }
    }

    public function registerDatapocketRoute($url)
    {
        if (!$url) {
            return json_encode([
                'status' => 'error',
                'message' => $this->l('The webhook route is missing'),
            ]);
        }

        Db::getInstance()->insert(
            'datapocket_routes',
            [
                'url' => $url
            ]
        );
        $idDatapocketRoutes = (int)Db::getInstance()->Insert_ID();

        return json_encode([
            'status' => 'success',
            'webhookid' => $idDatapocketRoutes,
            'url' => $url
        ]);
    }

    public function updateDatapocketRoute($id, $url)
    {
        if (!$id) {
            return json_encode([
                'status' => 'error',
                'message' => $this->l('The webhook ID is missing'),
            ]);
        }

        if (!$url) {
            return json_encode([
                'status' => 'error',
                'message' => $this->l('The webhook route is missing'),
            ]);
        }

        $sql = 'SELECT `id_datapocket_routes`
                FROM `' . _DB_PREFIX_ . 'datapocket_routes`
                WHERE `id_datapocket_routes` = ' . $id;

        $id_datapocket_routes = Db::getInstance()->getValue($sql);

        if (!$id_datapocket_routes) {
            return json_encode([
                'status' => 'error',
                'message' => $this->l('The webhook ID does not exist'),
                'webhookid' => $id
            ]);
        }

        $update = Db::getInstance()->update('datapocket_routes', [
            'url' => pSQL($url),
        ], 'id_datapocket_routes = ' . (int)$id);

        if ($update) {
            return json_encode([
                'status' => 'success',
                'message' => $this->l('The webhook has been modified'),
                'webhookid' => $id,
                'url' => $url
            ]);
        }
        return true;
    }

    public function checkDatapocketRoute($id)
    {
        if (!$id) {
            return json_encode([
                'status' => 'error',
                'message' => $this->l('The webhook ID is missing'),
                'webhookid' => $id
            ]);
        }

        $sql = 'SELECT `url`
                FROM `' . _DB_PREFIX_ . 'datapocket_routes`
                WHERE `id_datapocket_routes` = ' . $id;

        $url = Db::getInstance()->getValue($sql);

        if (!$url) {
            return json_encode([
                'status' => 'error',
                'message' => $this->l('The webhook ID does not exist'),
                'id' => $id
            ]);
        }

        return json_encode([
            'status' => 'success',
            'webhookid' => $id,
            'url' => $url
        ]);
    }

    public function deleteDatapocketRoute($id)
    {
        if (!$id) {
            return json_encode([
                'status' => 'error',
                'message' => $this->l('The webhook ID is missing'),
            ]);
        }

        $sql = 'SELECT `id_datapocket_routes`
                FROM `' . _DB_PREFIX_ . 'datapocket_routes`
                WHERE `id_datapocket_routes` = ' . $id;

        $idDatapocketRoutes = Db::getInstance()->getValue($sql);

        if (!$idDatapocketRoutes) {
            return json_encode([
                'status' => 'error',
                'message' => $this->l('The webhook ID does not exist'),
                'webhookid' => $id
            ]);
        }

        Db::getInstance()->delete(
            'datapocket_routes',
            'id_datapocket_routes = ' . (int)$idDatapocketRoutes
        );

        return json_encode([
            'status' => 'success',
            'message' => $this->l('The webhook was deleted'),
        ]);
    }

    public function hookActionProductUpdate($params)
    {
        $opt['resource'] = 'products';
        $opt['id'] = $params['product']->id;
        $xml = $this->webService->get($opt);
        $asXml = $xml->asXML();
        $method = "PUT";
        $object = "product";
        $indexed = (string)$xml->product->indexed;

        if ($indexed == 0) {
            $this->contactAPI($asXml, $method, $object);
        }
    }

    public function hookActionProductDelete($params)
    {
        $xml = new SimpleXMLElement('<?xml version="1.0" encoding="UTF-8"?><prestashop  xmlns:xlink="http://www.w3.org/1999/xlink" />');
        $element = $xml->addChild('product');
        $element->addChild('id', $params['product']->id);
        $element->addChild('status', "removed");
        $method = 'DELETE';
        $object = 'product';

        $dom = new DOMDocument("1.0");
        $dom->preserveWhiteSpace = false;
        $dom->formatOutput = true;
        $dom->loadXML($xml->asXML());

        $this->contactAPI($dom->saveXML(), $method, $object);
    }

    public function hookActionCategoryAdd($params)
    {
        $opt['resource'] = 'categories';
        $opt['id'] = $params['category']->id;

        $xml = $this->webService->get($opt);
        $xml = $xml->asXML();
        $method = 'POST';
        $object = 'category';
        $this->contactAPI($xml, $method, $object);
    }

    public function hookActionCategoryUpdate($params)
    {
        $opt['resource'] = 'categories';
        $opt['id'] = $params['category']->id;

        $xml = $this->webService->get($opt);
        $xml = $xml->asXML();
        $method = 'PUT';
        $object = 'category';
        $this->contactAPI($xml, $method, $object);
    }

    public function hookActionCategoryDelete($params)
    {
        $xml = new SimpleXMLElement('<?xml version="1.0" encoding="UTF-8"?><prestashop  xmlns:xlink="http://www.w3.org/1999/xlink" />');
        $element = $xml->addChild('category');
        $element->addChild('id', $params['category']->id);
        $element->addChild('status', "removed");
        $method = 'DELETE';
        $object = 'category';

        $dom = new DOMDocument("1.0");
        $dom->preserveWhiteSpace = false;
        $dom->formatOutput = true;
        $dom->loadXML($xml->asXML());

        $this->contactAPI($dom->saveXML(), $method, $object);
    }

    public function hookActionObjectSpecificPriceAddAfter($params)
    {
        $id = $params['object']->id;

        Db::getInstance()->insert(
            'datapocket_objects',
            [
                'id' => $id,
                'object' => 'discount',
                'status' => 'created',
                'sent' => 0,
            ]
        );
    }

    public function hookActionObjectSpecificPriceDeleteAfter($params)
    {
        $id = $params['object']->id;


        $sql = 'SELECT `id` FROM `' . _DB_PREFIX_ . 'datapocket_objects`
                WHERE `id` = ' . (int) $id . ' AND `object` = "discount"';
        $exist = Db::getInstance()->getValue($sql);

        if ($exist) {
            Db::getInstance()->update('datapocket_objects', [
                'status' => 'deleted',
            ], 'sent = 1 AND `id` = ' . (int) $id. ' AND `object` = "discount"');
        }

        $xml = new SimpleXMLElement('<?xml version="1.0" encoding="UTF-8"?><prestashop  xmlns:xlink="http://www.w3.org/1999/xlink" />');
        $discounts = $xml->addChild('SpecificPrices');

        $starting_discount = $discounts->addChild('starting');
        $ending_discount = $discounts->addChild('ending');

        $ending_discounts = $ending_discount->addChild('SpecificPrice');
        $ending_discounts->addChild('id', $id);

        $dom = new DOMDocument("1.0");
        $dom->preserveWhiteSpace = false;
        $dom->formatOutput = true;
        $dom->loadXML($xml->asXML());

        $method = 'DELETE';
        $object = '/specificprices';
        $this->contactAPI($dom->saveXML(), $method, $object);
    }

    public function hookActionObjectSpecificPriceUpdateAfter($params)
    {

        Db::getInstance()->update('datapocket_objects', [
            'status' => 'updated',
            'sent' => '0',
        ], '`id` = ' . (int) $params['object']->id. ' AND `object` = "discount"');
    }

    public function getDiscounts()
    {
        $sql = 'SELECT `id`, `status` FROM `' . _DB_PREFIX_ . 'datapocket_objects`
                WHERE `object` = "discount" AND `sent` = 0';

        $specificPrices = Db::getInstance()->executeS($sql);


        $xml = new SimpleXMLElement('<?xml version="1.0" encoding="UTF-8"?><prestashop  xmlns:xlink="http://www.w3.org/1999/xlink" />');
        $discounts = $xml->addChild('SpecificPrices');

        $starting_discount = $discounts->addChild('starting');
        $ending_discount = $discounts->addChild('ending');

        foreach ($specificPrices as $specificPrice) {
            $id = $specificPrice['id'];
            $discount = new SpecificPrice($id);

            if (!Validate::isLoadedObject($discount)) {
                continue;
            }

            $id_product = (int) $discount->id_product;
            $product = new Product($id_product);

            if (!Validate::isLoadedObject($product)) {
                continue;
            }

            $now = date("Y-m-d");
            $infinite = '0000-00-00';

            $from = explode(' ', $discount->from);
            $from = $from[0];

            $to = explode(' ', $discount->to);
            $to = $to[0];

            if ($to <= $now && $to != $infinite) {

                $ending_discounts = $ending_discount->addChild('SpecificPrice');
                $ending_discounts->addChild('id', $discount->id);
                $ending_discounts->addChild('id_product', $discount->id_product);
                $ending_discounts->addChild('id_product_attribute', $discount->id_product_attribute);
                $ending_discounts->addChild('id_shop', $discount->id_shop);
                $ending_discounts->addChild('id_shop_group', $discount->id_shop_group);
                $ending_discounts->addChild('id_currency', $discount->id_currency);
                $ending_discounts->addChild('id_country', $discount->id_country);
                $ending_discounts->addChild('id_group', $discount->id_group);
                $ending_discounts->addChild('id_customer', $discount->id_customer);
                $ending_discounts->addChild('price', $discount->price);
                $ending_discounts->addChild('from_quantity', $discount->from_quantity);
                $ending_discounts->addChild('reduction', $discount->reduction);
                $ending_discounts->addChild('reduction_tax', $discount->reduction_tax);
                $ending_discounts->addChild('reduction_type', $discount->reduction_type);
                $ending_discounts->addChild('from', $discount->from);
                $ending_discounts->addChild('to', $discount->to);

                Db::getInstance()->update('datapocket_objects', [
                    'status' => 'sent',
                    'sent' => '1',
                ], '`id` = ' . (int) $id. ' AND `object` = "discount"');
            } else if ($from == $infinite && $to == $infinite) {

                $starting_discounts = $starting_discount->addChild('SpecificPrice');
                $starting_discounts->addChild('id', $discount->id);
                $starting_discounts->addChild('id_product', $discount->id_product);
                $starting_discounts->addChild('id_product_attribute', $discount->id_product_attribute);
                $starting_discounts->addChild('id_shop', $discount->id_shop);
                $starting_discounts->addChild('id_shop_group', $discount->id_shop_group);
                $starting_discounts->addChild('id_currency', $discount->id_currency);
                $starting_discounts->addChild('id_country', $discount->id_country);
                $starting_discounts->addChild('id_group', $discount->id_group);
                $starting_discounts->addChild('id_customer', $discount->id_customer);
                $starting_discounts->addChild('price', $discount->price);
                $starting_discounts->addChild('from_quantity', $discount->from_quantity);
                $starting_discounts->addChild('reduction', $discount->reduction);
                $starting_discounts->addChild('reduction_tax', $discount->reduction_tax);
                $starting_discounts->addChild('reduction_type', $discount->reduction_type);
                $starting_discounts->addChild('from', $discount->from);
                $starting_discounts->addChild('to', $discount->to);

                Db::getInstance()->update('datapocket_objects', [
                    'status' => 'sent',
                    'sent' => '1',
                ], '`id` = ' . (int) $id. ' AND `object` = "discount"');
            } else if ($from == $now) {
                $starting_discounts = $starting_discount->addChild('SpecificPrice');
                $starting_discounts->addChild('id', $discount->id);
                $starting_discounts->addChild('id_product', $discount->id_product);
                $starting_discounts->addChild('id_product_attribute', $discount->id_product_attribute);
                $starting_discounts->addChild('id_shop', $discount->id_shop);
                $starting_discounts->addChild('id_shop_group', $discount->id_shop_group);
                $starting_discounts->addChild('id_currency', $discount->id_currency);
                $starting_discounts->addChild('id_country', $discount->id_country);
                $starting_discounts->addChild('id_group', $discount->id_group);
                $starting_discounts->addChild('id_customer', $discount->id_customer);
                $starting_discounts->addChild('price', $discount->price);
                $starting_discounts->addChild('from_quantity', $discount->from_quantity);
                $starting_discounts->addChild('reduction', $discount->reduction);
                $starting_discounts->addChild('reduction_tax', $discount->reduction_tax);
                $starting_discounts->addChild('reduction_type', $discount->reduction_type);
                $starting_discounts->addChild('from', $discount->from);
                $starting_discounts->addChild('to', $discount->to);

                Db::getInstance()->update('datapocket_objects', [
                    'status' => 'sent',
                    'sent' => '1',
                ], '`id` = ' . (int) $id. ' AND `object` = "discount"');
            } else if ($to <= $now && $to == $infinite) {
                $starting_discounts = $starting_discount->addChild('SpecificPrice');
                $starting_discounts->addChild('id', $discount->id);
                $starting_discounts->addChild('id_product', $discount->id_product);
                $starting_discounts->addChild('id_product_attribute', $discount->id_product_attribute);
                $starting_discounts->addChild('id_shop', $discount->id_shop);
                $starting_discounts->addChild('id_shop_group', $discount->id_shop_group);
                $starting_discounts->addChild('id_currency', $discount->id_currency);
                $starting_discounts->addChild('id_country', $discount->id_country);
                $starting_discounts->addChild('id_group', $discount->id_group);
                $starting_discounts->addChild('id_customer', $discount->id_customer);
                $starting_discounts->addChild('price', $discount->price);
                $starting_discounts->addChild('from_quantity', $discount->from_quantity);
                $starting_discounts->addChild('reduction', $discount->reduction);
                $starting_discounts->addChild('reduction_tax', $discount->reduction_tax);
                $starting_discounts->addChild('reduction_type', $discount->reduction_type);
                $starting_discounts->addChild('from', $discount->from);
                $starting_discounts->addChild('to', $discount->to);

                Db::getInstance()->update('datapocket_objects', [
                    'status' => 'sent',
                    'sent' => '1',
                ], '`id` = ' . (int) $id. ' AND `object` = "discount"');

            } else if ($from <= $now && $to >= $now) {
                $starting_discounts = $starting_discount->addChild('SpecificPrice');
                $starting_discounts->addChild('id', $discount->id);
                $starting_discounts->addChild('id_product', $discount->id_product);
                $starting_discounts->addChild('id_product_attribute', $discount->id_product_attribute);
                $starting_discounts->addChild('id_shop', $discount->id_shop);
                $starting_discounts->addChild('id_shop_group', $discount->id_shop_group);
                $starting_discounts->addChild('id_currency', $discount->id_currency);
                $starting_discounts->addChild('id_country', $discount->id_country);
                $starting_discounts->addChild('id_group', $discount->id_group);
                $starting_discounts->addChild('id_customer', $discount->id_customer);
                $starting_discounts->addChild('price', $discount->price);
                $starting_discounts->addChild('from_quantity', $discount->from_quantity);
                $starting_discounts->addChild('reduction', $discount->reduction);
                $starting_discounts->addChild('reduction_tax', $discount->reduction_tax);
                $starting_discounts->addChild('reduction_type', $discount->reduction_type);
                $starting_discounts->addChild('from', $discount->from);
                $starting_discounts->addChild('to', $discount->to);

                Db::getInstance()->update('datapocket_objects', [
                    'status' => 'sent',
                    'sent' => '1',
                ], '`id` = ' . (int) $id. ' AND `object` = "discount"');
            }
        }

        $dom = new DOMDocument("1.0");
        $dom->preserveWhiteSpace = false;
        $dom->formatOutput = true;
        $dom->loadXML($xml->asXML());

        $method = 'POST';
        $object = '/specificprices';
        $this->contactAPI($dom->saveXML(), $method, $object);

        return $discounts;

    }

    private function contactAPI($data, $method, $object)
    {
        $sql = 'SELECT `url`
                FROM `' . _DB_PREFIX_ . 'datapocket_routes`';

        $urls = Db::getInstance()->executeS($sql);
        foreach ($urls as $url) {
            if (!empty($object)) {
                $url = $url['url'] . '/' . $object . '/';
            } else {
                $url = $url['url'];
            }
            ob_start();
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_HTTPHEADER, array(
                'Content-type: text/xml'
            ));
            curl_setopt($ch, CURLOPT_URL, $url);
            curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $method);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
            @curl_exec($ch);
            curl_close($ch);
            ob_end_clean();
        }

    }

    public function hookActionProductSave($params)
    {
        $id = $params['product']->id;
        $opt['resource'] = 'products';
        $opt['id'] = $id;

        $sql = 'SELECT `id` FROM `' . _DB_PREFIX_ . 'datapocket_objects`
                WHERE `id` = ' . (int) $id . ' AND `object` = "product"';
        $exist = Db::getInstance()->getValue($sql);

        if (!$exist) {
            $xml = $this->webService->get($opt);
            $xml = $xml->asXML();
            $method = 'POST';
            $object = 'product';
            $this->contactAPI($xml, $method, $object);

            Db::getInstance()->insert(
                'datapocket_objects',
                [
                    'id' => $id,
                    'object' => 'product',
                    'status' => 'created',
                ]
            );
            return true;
        }

        return true;
    }
}
